/*
 ***********************************************************************
 *
 *				Simple 3D graphics with animation
 *						     Root module
 *
 ***********************************************************************
 */

/* MacHeaders Included */
#include <oops.h>
#include "a_menu.h"
#include "a_window.h"
#include <stdio.h>

#define DATABASE_INCLUDE 1
#include "Database.h"

/*
 *----------------------------------------------------------------------
 *							The life is going on here
 */
 
class TopBoss : public AWindow
{
public:
	TopBoss(void);
	~TopBoss(void);
	virtual void click(void);				// Handle a click event
	virtual void timeclick(void);			// Handle a time click
};

AWindow * A_window = (void *)0;

TopBoss::TopBoss(void)
{
	display();
}

TopBoss::~TopBoss(void)
{
}

void TopBoss::click(void)
{
  display();
}

void TopBoss::timeclick(void)
{
  static long int prev_tick_count;
  
  if( !TD_animation )
    return;
    
  if( TickCount() - prev_tick_count > TD_animation_period )
    prev_tick_count = TickCount(),
    click();
}

/*
 *----------------------------------------------------------------------
 *							File Menu
 */

enum { NewItem=1, OpenItem, CloseItem, QuitItem=CloseItem+2 };
class FileMenu : public AMenu
{
public:
	FileMenu(void);
	~FileMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

FileMenu::FileMenu(void)
{
	new_menu("\pFile");
	AppendMenu(handle, "\pNew/N;Open/O;Close/W;(-;Quit/Q");
}

FileMenu::~FileMenu(void)
{
}

void FileMenu::Adjust(void)
{
	if( A_window == (void *)0 )
	{
	  EnableItem(handle,NewItem);
	  EnableItem(handle,OpenItem);
	  DisableItem(handle,CloseItem);
	}
	else
	{
	  DisableItem(handle,NewItem);
	  DisableItem(handle,OpenItem);
	  EnableItem(handle,CloseItem);
	}
}

void FileMenu::Handle(const short menuItem)
{
	switch (menuItem)
	{
	  case  NewItem:
	  case  OpenItem:
	  		A_window = new Life;
	  		break;
	  		
	  case	CloseItem:
 			delete A_window;
 			A_window = (void *)0;
  			break;
  				  	
	  case	QuitItem:
			ExitToShell();
			break;
	}
}

/*
 *----------------------------------------------------------------------
 *							    Size Menu
 */

class SizeMenu : public AMenu
{
	short scales[4];						// Array of possible scales
	
public:
	SizeMenu(void);
	~SizeMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

SizeMenu::SizeMenu(void)
{
	register int i;
	char buffer[40];
	scales[0] = 1;
	scales[1] = 2;
	scales[2] = 3;
	scales[3] = 4;

	new_menu("\pSize");
	for(i=0; i<sizeof(scales)/sizeof(scales[0]); i++)
	{
	   sprintf(buffer,"Scale x%ld",scales[i]);
	   CtoPstr(buffer);
	   AppendMenu(handle,buffer);
	}
}

SizeMenu::~SizeMenu(void)
{
}

void SizeMenu::Adjust(void)
{
	register int i;
	for(i=0; i<sizeof(scales)/sizeof(scales[0]); i++)
	  if( scales[i] == Display_scale_factor )
	    CheckItem(handle,i+1,TRUE);
}

void SizeMenu::Handle(const short menu_item)
{	
	register int i;
	if( menu_item >= 1 && menu_item <= sizeof(scales)/sizeof(scales[0]) )
	{
	  for(i=0; i<sizeof(scales)/sizeof(scales[0]); i++)
	     if( scales[i] == Display_scale_factor )
	       CheckItem(handle,i+1,FALSE);
	  Display_scale_factor = scales[menu_item-1];
	}
	 
	if( A_window != (void *)0 )
	{
	  delete A_window;
	  A_window = new Life;
	}
} 

/*
 *----------------------------------------------------------------------
 *							    Modes Menu
 */

enum {ModesAutoItem=1, ModesLifeItem=ModesAutoItem+2, ModesJuliaItem};

class ModesMenu : public AMenu
{
	
public:
	ModesMenu(void);
	~ModesMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

ModesMenu::ModesMenu(void)
{
	new_menu("\pModes");
	AppendMenu(handle,"\pAnimate;(-;Life;Julia set");
}

ModesMenu::~ModesMenu(void)
{
}

void ModesMenu::Adjust(void)
{
	CheckItem(handle,ModesAutoItem,Life_animation);
	CheckItem(handle,ModesLifeItem,Program_mode == LIFE);
	CheckItem(handle,ModesJuliaItem,Program_mode == JULIA);
}

void ModesMenu::Handle(const short menu_item)
{	
	switch (menu_item)
	{
	  case  ModesAutoItem:
	  		TD_animation = !TD_animation;
	  		break;
	  		
	  case  ModesLifeItem:
	  		Program_mode = LIFE;
	  		if( A_window != (void *)0 )
	  		  delete A_window;
	  		A_window = new Life;
	  		break;

	  case  ModesJuliaItem:
	  		Program_mode = JULIA;
	  		if( A_window != (void *)0 )
	  		  delete A_window;
	  		A_window = new Life;
	  		break;
	}
} 


/*
 *----------------------------------------------------------------------
 *							Initialization, etc.
 */


/*
 *----------------------------------------------------------------------
 *						This is where everything happens
 */

extern void HandleEvent(void);

main()
{ 
	FileMenu * a_FileMenu;
	SizeMenu * a_SizeMenu;
	ModesMenu * a_ModesMenu;
	
	Initialize_MAC();
	A_menu_init();

	a_FileMenu = new FileMenu;
	a_SizeMenu = new SizeMenu;
	a_ModesMenu = new ModesMenu;
	
	A_window = new TopBoss;
		
	for (;;)
		HandleEvent();
}
