/*
 ***********************************************************************
 *
 *				Simple 3D graphics with animation
 *						     Root module
 *
 ***********************************************************************
 */

/* MacHeaders Included */
#include <oops.h>
#include "a_menu.h"
#include "a_dialog.h"
#include "Figures.h"
#include <stdio.h>

#define DATABASE_INCLUDE 1
#include "Database.h"

/*
 *----------------------------------------------------------------------
 *							Opening New windows
 */

AWindow * A_window = (void *)0;

static void new_figure(void)
{
	switch(Picture_type)
	{
	  case CUBE:
		   A_window = new Cube;
		   break;

	  case PYRAMID:
		   A_window = new Pyramid;
		   break;

	  case PYRAMIDS:
		   A_window = new Pyramids;
		   break;

	  default:
	       assure(0,"may not happen");
	}
}

/*
 *----------------------------------------------------------------------
 *							File Menu
 */

enum { NewItem=1, OpenItem, CloseItem, QuitItem=CloseItem+2 };
class FileMenu : public AMenu
{
public:
	FileMenu(void);
	~FileMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

FileMenu::FileMenu(void)
{
	new_menu("\pFile");
	AppendMenu(handle, "\pNew/N;Open/O;Close/W;(-;Quit/Q");
}

FileMenu::~FileMenu(void)
{
}

void FileMenu::Adjust(void)
{
	if( A_window == (void *)0 )
	{
	  EnableItem(handle,NewItem);
	  EnableItem(handle,OpenItem);
	  DisableItem(handle,CloseItem);
	}
	else
	{
	  DisableItem(handle,NewItem);
	  DisableItem(handle,OpenItem);
	  EnableItem(handle,CloseItem);
	}
}

void FileMenu::Handle(const short menuItem)
{
	switch (menuItem)
	{
	  case  NewItem:
	  case  OpenItem:
	  		new_figure();
	  		break;
	  		
	  case	CloseItem:
 			delete A_window;
 			A_window = (void *)0;
  			break;
  				  	
	  case	QuitItem:
			ExitToShell();
			break;
	}
}


/*
 *----------------------------------------------------------------------
 *							Projection Modes Menu
 */

enum {ModesDimetricItem=1, ModesIsometricItem, ModesProspectiveItem};

class ProjectionModesMenu : public AMenu
{
	
public:
	ProjectionModesMenu(void);
	~ProjectionModesMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

ProjectionModesMenu::ProjectionModesMenu(void)
{
	new_menu("\pProjection");
	AppendMenu(handle,"\pDimetric;Isometric;Prospective");
}

ProjectionModesMenu::~ProjectionModesMenu(void)
{
}

void ProjectionModesMenu::Adjust(void)
{
	CheckItem(handle,ModesDimetricItem,Projection_mode == DIMETRIC);
	CheckItem(handle,ModesIsometricItem,Projection_mode == ISOMETRIC);
	CheckItem(handle,ModesProspectiveItem,Projection_mode == PROSPECTIVE);
}

enum { PDItem_apply = 1, PDItem_YZSet = 2, PDItem_Xval = 5, PDItem_cancel = 7 };
 
class PerspectiveProjDialog : public Dialog
{
public:
	PerspectiveProjDialog(void);
	~PerspectiveProjDialog(void);
	virtual void hit(const int item_hit);		// Handle hitting an item
};

PerspectiveProjDialog::PerspectiveProjDialog(void)
{
	open_dialog(128);
	set_default_item(1);
	select_whole_text(5);
}

PerspectiveProjDialog::~PerspectiveProjDialog(void)
{
}

void PerspectiveProjDialog::hit(const int item_hit)
{
	long proj_x;
	switch(item_hit)
	{
	  case PDItem_YZSet:
	  	   if( A_window != (void *)0 )
	  	   {
	  	      Point point;
	  	   	  close();				// Note, the dialog needs to be closed before
	  	   	  						// picking up the point. Event manager in
	  	   	  						// pick_point doesn't work properly when the
	  	   	  						// dialog window is open
	  	      point = ((SpacePoints*)A_window)->pickup_point_from_window();
	  	      ((SpacePoints*)A_window)->S.y = point.h;
	  	      ((SpacePoints*)A_window)->S.z = point.v;
	  	     (*A_window).refresh();
	  	   }
	  	   new PerspectiveProjDialog;
	  	   break;

	  case PDItem_Xval:
	  	   break;

	  case PDItem_apply:
	  	   StringToNum(get_text(5),&proj_x);
	  	   if( A_window != (void *)0 )
	  	   {
	  	     ((SpacePoints*)A_window)->S.x = proj_x;
	  	     (*A_window).refresh();
	  	   }
	  	   else
	  	     new_figure();
		   select_whole_text(5);
	  	   break;

	  case PDItem_cancel:
	  	   close();
	  	   break;
	}
}

void ProjectionModesMenu::Handle(const short menu_item)
{	
	switch (menu_item)
	{
	  case  ModesDimetricItem:
	  		Projection_mode = DIMETRIC;
	  		(*A_window).refresh();
	  		break;

	  case  ModesIsometricItem:
	  		Projection_mode = ISOMETRIC;
	  		(*A_window).refresh();
	  		break;

	  case  ModesProspectiveItem:
	  		Projection_mode = PROSPECTIVE;
	  		new PerspectiveProjDialog;
	  		(*A_window).refresh();
	  		break;
	}
} 

/*
 *----------------------------------------------------------------------
 *							    Picture type Menu
 */

enum {PictureCubeItem=1, PicturePyramidItem, PicturePyramidsItem};

class PictureTypeMenu : public AMenu
{
	
public:
	PictureTypeMenu(void);
	~PictureTypeMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

PictureTypeMenu::PictureTypeMenu(void)
{
	new_menu("\pPictures");
	AppendMenu(handle,"\pCube;Pyramid;Pyramids");
}

PictureTypeMenu::~PictureTypeMenu(void)
{
}

void PictureTypeMenu::Adjust(void)
{
	CheckItem(handle,PictureCubeItem,Picture_type == CUBE);
	CheckItem(handle,PicturePyramidItem,Picture_type == PYRAMID);
	CheckItem(handle,PicturePyramidsItem,Picture_type == PYRAMIDS);
}

void PictureTypeMenu::Handle(const short menu_item)
{	
	Picture_type = menu_item - 1;
	if( A_window != (void *)0 )
	  delete A_window;
	new_figure();
}

/*
 *----------------------------------------------------------------------
 *							   Rotation mode Menu
 */

enum {RotationWithMouseItem=1, RotationSetPointItem, Delimiter, RotationAnimateItem};

class RotationModeMenu : public AMenu
{
	
public:
	RotationModeMenu(void);
	~RotationModeMenu(void);
	
	virtual void Handle(const short item_no);		// Handle an item
	virtual void Adjust(void);						// Adjust the items in the menu
};

RotationModeMenu::RotationModeMenu(void)
{
	new_menu("\pRotation");
	AppendMenu(handle,"\pWith mouse;Set point;(-;Animate");
}

RotationModeMenu::~RotationModeMenu(void)
{
}

void RotationModeMenu::Adjust(void)
{
	if( A_window == (void *)0 )
	{
	  DisableItem(handle,RotationWithMouseItem);
	  DisableItem(handle,RotationSetPointItem);
	  DisableItem(handle,RotationAnimateItem);
	}
	else
	{
	  EnableItem(handle,RotationWithMouseItem);
	  EnableItem(handle,RotationSetPointItem);
	  EnableItem(handle,RotationAnimateItem);
	  CheckItem(handle,RotationAnimateItem,TD_animation);
	}
}

void RotationModeMenu::Handle(const short menu_item)
{	
	switch (menu_item)
	{
	  case  RotationWithMouseItem:
	  		TD_rotate_with_mouse = TRUE;
	  		break;

	  case  RotationSetPointItem:
	  	   if( A_window != (void *)0 )
	  	   {
	  	      Point point;
	  	      point = ((SpacePoints*)A_window)->pickup_point_from_window();
	  	      ((StirringSpacialPoints*)A_window)->
	  	      	set_rotation_point1(0,point.h,point.v);
	  	     (*A_window).refresh();
	  	    }
	  		break;

	  case  RotationAnimateItem:
	  		TD_animation = !TD_animation;
	  		(*A_window).refresh();
	  		break;
	}
}


/*
 *----------------------------------------------------------------------
 *							Initialization, etc.
 */


/*
 *----------------------------------------------------------------------
 *						This is where everything happens
 */

extern void HandleEvent(void);

main()
{ 
	FileMenu * a_FileMenu;
	ProjectionModesMenu * a_ProjectionModesMenu;
	PictureTypeMenu * a_PictureTypeMenu;
	RotationModeMenu * a_RotationModeMenu;
	
	Initialize_MAC();
	A_menu_init();

	a_FileMenu = new FileMenu;
	a_ProjectionModesMenu = new ProjectionModesMenu;
	a_PictureTypeMenu = new PictureTypeMenu;
	a_RotationModeMenu = new RotationModeMenu;
	
	new_figure();
		
	for (;;)
		HandleEvent();
}
