/*
 ***********************************************************************
 *
 *					 			Set of 3D Points
 *
 * The present file defines classes to handle a set of 3D points, i.e.
 * to display them using the appropritae projection and perform some
 * linear space transformation (rotation, translation, etc).
 *
 ***********************************************************************
 */

#pragma once				// include this header only once per source file

#include "a_window.h"

											// A point into space
typedef struct
{
	short x, y, z;
} SpacePoint;

class SpacePoints : public AWindow
{
protected:
	short npoints;							// No of points in the set
	SpacePoint * points;					// Array of points

private:
	Point * proj_points;					// Array of 2D points - plane projections
	PicHandle	  picture;
	Rect	proj_rect;						// Specifies the area of plane we're messing with
    
	void project(void);						// Project 3D points to the plane
	virtual void draw(void);				// Drawing procedure in our window

public:
    SpacePoint S;							// Center of the prospective projection
   
	SpacePoints(void);
	~SpacePoints(void);
	void allocate(const short _npoints);	// Allocate the set
											// Set the value of a SpacePoint
											// Note the points are numbered starting from 1
	void set(const short point_no, const short x, const short y, const short z);
	void set_proj_rect(const short left, const short top, const short right, const short bottom);
	Point pickup_point_from_window(void);
	virtual void refresh(void);				// Redisplay the set

protected:
											// The functions below are to be used
											// in drawing the picture. They must be called
											// by draw() or its descendants

											// Connect point1 with point2, etc.
											// Terminate the list with 0
	void line_segments(const short point1, const short point2,...);
	void color(const short color_no);		// Select the color
	virtual void draw_picture(void);		// That draws a picture using the set and
											// functions defined above
};
