/*
 ***********************************************************************
 *
 *						    Dialog generic interface
 *
 * The present file provides generic interfaces to the display manager
 * Derived classes should modify some basis functions depending on the
 * specific dialog. 
 *
 ***********************************************************************
 */

#include "a_dialog.h"

/*
 *----------------------------------------------------------------------
 *						  Constructing and Destructing
 */

Dialog * Active_dialog = (void *)0;

									// Derived classes should call this first,
									// and then provide smth more meaningful
Dialog::Dialog(void)
{
	the_dialog = (void *)0;
	Active_dialog = this;
	default_item = 0;
}

									// Real creator of the dialog. It is to be called
									// from within the derived class. The derived class
									// then needs to fill up dilog fields if needed
void Dialog::open_dialog(const int dialog_resource_no)
{
	assert( the_dialog == (void *)0 );
	the_dialog = GetNewDialog(dialog_resource_no,(void *)0,(void *)-1);
	assert( the_dialog != (void *)0 );
}

void Dialog::set_default_item(const short _default_item)
{
	default_item = _default_item;
}

Dialog::~Dialog(void)
{
	Active_dialog = (void *)0;
	DisposDialog(the_dialog);
}

void Dialog::close(void)
{
	delete this;						// This implicitly calls destructor
}

/*
 *----------------------------------------------------------------------
 *						  	Handle the event in the dialog
 */

										// The following procedure is called by the
										// Event manager if there is a suspicion that
										// something happened with our dialog
void Dialog::handle(const EventRecord * event)
{
	short itemhit;
	DialogPtr a_dialog;
	static int count = 0;
		
											// Hit default item (if any) when Return
											// key is pressed
	if( event->what == keyDown && (event->message & charCodeMask) == 0x0d &&
	    default_item > 0 )
	{
	  hit(default_item);
	  return;
	}
	  
	if( DialogSelect(event,&a_dialog,&itemhit) && a_dialog == the_dialog )
	  if( event->what == mouseDown )		// Mouse down is always worth taking
	  		  hit(itemhit);					// some actions
}

void Dialog::hit(const int item_hit)
{
	_error("Derived class should've handle hitting an item");
}

										// Select entire text in the text item
void Dialog::select_whole_text(const int item_no)
{
	SelIText(the_dialog,item_no,0,255);
}

										// Return the contents of the text item
Str255 * Dialog::get_text(const int item_no)
{
	short  item_type;
	Handle item_handle;
	Rect   item_box;
	static Str255 item_text;
	
	GetDItem(the_dialog,item_no,&item_type,&item_handle,&item_box);
	assure(item_handle != (void *)0, "Invalid item_no in get_text");
	GetIText(item_handle,&item_text);
	return &item_text;
}
