/*
 ***********************************************************************
 *
 *			    Menu generic object
 *
 * Implementation of the very general actions to handle a menu.
 * Handling the Apple Menu.
 ***********************************************************************
 */

#include <oops.h>
#include "a_menu.h"
#pragma implementation

AMenu * AMenu::menus[Max_no_menus];
short AMenu::no_menus = 0;

/*
 *----------------------------------------------------------------------
 *			Constructors and Destructors
 */

void AMenu::new_menu(Str255 name)
{
	const short menu_id = no_menus+1;		// Numeration from 1
	
	if( no_menus >= Max_no_menus )
	  _error("Too many menus!");
	InsertMenu(handle = NewMenu(menu_id,name), 0);
	menus[no_menus++] = this;
	DrawMenuBar();
}

AMenu::~AMenu(void)
{
}

/*
 *----------------------------------------------------------------------
 *		   Generic Adjusting the menu item to
 *		 the environment that might have changed
 */

void AMenu::Adjust(void)
{
}

				// Adjust through all the menus in the bar
void AMenu::AdjustAll(void)
{
	short	menu_id;

	for(menu_id=1; menu_id <= no_menus; menu_id++)
	  (*menus[menu_id-1]).Adjust();
}


/*
 *----------------------------------------------------------------------
 *		Generic Handling of a selected item
 */

void AMenu::Handle(const short item_no)
{
					// Subclasses must do something here
	_error("Generic Handle - no specific Menu Handling defined!");
}

			// Handle the event of menu selection
			// mSelect is what MenuSelect() and
			// MenuKey() return: the high word is the menu ID, the low word
			// is the menu item
void AMenu::MenuSelect(long mSelect)
{
	short	menu_id = mSelect >> 16;
	short	menu_item = mSelect & 0xffff;

//	message("mSelect is %lx,Menu ID is %ld, menu item is %ld\n",mSelect,menu_id,menu_item);
	if( menu_id == 0 )	// The user pulled down a menu but later
	  return;		// has changed his mind
	if( menu_id <= 0 )
	  _error("Menu ID is not positive!");
	  
	if( menu_id > no_menus )
	  _error("Menu ID of the selected menu is larger the no of menus!");
	  
	(*menus[menu_id-1]).Handle(menu_item);
}


/*
 *----------------------------------------------------------------------
 *			Handling an Apple Menu
 */

					// Constructor
AppleMenu::AppleMenu(void)
{
	new_menu("\p\024");
	AddResMenu(handle, 'DRVR');
}

AppleMenu::~AppleMenu(void)
{
}

void AppleMenu::Handle(const short menuItem)
{
	GrafPtr		savePort;
	Str255		name;

	if( menuItem == 1 )
	{											// Item "About..." has been selected
	  Alert(128, 0L);
	  return;
	}
	 											// DA application item has been
	 											// selected from the Apple menu
	GetPort(&savePort);
	GetItem(handle, menuItem, name);
	OpenDeskAcc(name);
	SetPort(savePort);
}

AppleMenu * The_Apple_menu;

/*
 *----------------------------------------------------------------------
 *				Packet Initialization
 */

void A_menu_init(void)
{
	The_Apple_menu = new AppleMenu;
}

#if 0
/*
 *----------------------------------------------------------------------
 *			Handling the Menu Bar
 */

					// Constructor
MenuBar::MenuBar(void) : max_no_menus(5)
{
	no_menus = 0;
	handles = new MenuHandle[max_no_menus];
}

					// Destructor
MenuBar::~MenuBar(void)
{
}

					// Add a menu to the menu bar
void MenuBar::add(MenuHandle& handle)
{
	if( no_menus >= max_no_menus )
	  DebugStr("\pToo many menus!");
	handles[no_menu++] = &handle;
}

					// Handle the event of menu selection
					// mSelect is what MenuSelect() and
					// MenuKey() return: the high word is the menu ID, the low word
					// is the menu item
void MenuBar::Handle(long mSelect)
{
	short			menu_id = HiWord(mSelect);
	short			menu_item = LoWord(mSelect);

	if( menu_id <= 0 )
	  DebugStr("\pMenu ID is not positive!");
	  
	if( menu_id > no_menus )
	  DebugStr("\pMenu ID of the selected menu is larger the no of menus!");
	  
	 handles[menu_id-1].Handle(menu_item);
}
#endif
