/*
 ***********************************************************************
 *
 *						    Display manager interface
 *
 * The present file provides interfaces to the display manager who is
 * the only one guy who knows where is our window and what and how to
 * display in it
 *
 ***********************************************************************
 */

#include "a_window.h"
#include "Database.h"

/*
 *----------------------------------------------------------------------
 *						Construction of a window
 */

const int Window_resource_id = 128;

static pascal void ProcessPictureComment(short kind, short add_data_size, Handle add_data);

void AWindow::AWindow(void)
{
	Rect bounds_rectangle;
	static QDProcs qd_procs;
	monitor_mouse = (void *)0;
	
	the_window = GetNewWindow(Window_resource_id,(void *)0,(void *)(-1));
	assert( the_window != 0L );
	
	SetStdProcs(&qd_procs);
	qd_procs.commentProc = (void *)ProcessPictureComment;
	the_window->grafProcs = &qd_procs;
	
	SetPort(the_window);
	SelectWindow(the_window);
}

void AWindow::~AWindow(void)
{
	assert( the_window != 0 );
	DisposeWindow(the_window);
	the_window = 0;
}


/*
 *----------------------------------------------------------------------
 *							Handle some window events
 */

void AWindow::close(void)
{
	delete this;						// Implicitly calls the destructor
}

void AWindow::update(void)
{
	GrafPtr	savePort;
	
	GetPort(&savePort);
	SetPort(the_window);
	BeginUpdate(the_window);
	draw();
	EndUpdate(the_window);
	SetPort(savePort);
}

									// Drag the window to a new location
void AWindow::drag(Point where)
{
	DragWindow(the_window,where,&screenBits.bounds);
}

void AWindow::grow(Point where)
{
	long size;
	size = GrowWindow(the_window,where,&screenBits.bounds);
	if( size == 0 )
	  return;							// Window size hasn't actually been changed

										// Erase the scroll area	
	EraseRect(&the_window->portRect);
	SizeWindow(the_window,LoWord(size),HiWord(size),TRUE);	// Resize the window
	refresh();
}


										// Handle an activate or deactivate event
void AWindow::activate (const short go_active)
{
//	PenNormal();
	DrawGrowIcon(the_window);
		
	if (go_active)
	  SetPort(the_window);
}

void AWindow::refresh(void)
{
	EraseRect(&the_window->portRect);
	InvalRect(&the_window->portRect);			// Make the entire window being redrawn
}

void AWindow::select(void)
{
	SelectWindow(the_window);
}

												// True if some_window is our
int AWindow::is_our_window(const WindowPtr some_window)
{
	if( some_window == the_window )
	  return TRUE;
	else
	  return FALSE;
}

												// Figure out if the point in global
												// coordinates lies within our window
int AWindow::is_point_within_the_window(const Point global_point)
{
	WindowPtr port_save;
	int result;
	GetPort(&port_save);
	result = PtInRect(global_point,&the_window->portRect);
	SetPort(port_save);
	return result;
}

											// Region that defines the window contents
											// in global coordinates. Don't forget to dispose
											// the handle
RgnHandle AWindow::window_inside_region(void)
{
	RgnHandle window_rgn = NewRgn();
	RectRgn(window_rgn,&the_window->portRect);
	OffsetRgn(window_rgn,-(the_window->portBits.bounds.left),
						 -(the_window->portBits.bounds.top));
	return window_rgn;
}


void AWindow::click(void)
{
	_error("Derived class should have redefined it!");
}

void AWindow::timeclick(void)
{
}

/*
 *----------------------------------------------------------------------
 *								Actually displaying
 */

static long Palette[] =
{
	whiteColor, greenColor, redColor, blueColor, cyanColor, 
	magentaColor, yellowColor, blackColor
};

									// It just sets the foreground color
static pascal void ProcessPictureComment(short kind, short add_data_size, Handle add_data)
{
	ForeColor(Palette[kind]);	
}

void AWindow::draw(void)
{
}
